@@ -20,8 +20,9 @@ import (
 
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
+	routev1 "github.com/openshift/api/route/v1"
 
-	iov1 "github.com/openshift/cluster-ingress-operator/pkg/api/v1"
+	iov1 "github.com/openshift/api/operatoringress/v1"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
 	operatorclient "github.com/openshift/cluster-ingress-operator/pkg/operator/client"
 	"github.com/openshift/cluster-ingress-operator/pkg/operator/controller"
@@ -57,7 +58,7 @@ var (
 		{Type: operatorv1.LoadBalancerReadyIngressConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.DNSManagedIngressConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.DNSReadyIngressConditionType, Status: operatorv1.ConditionTrue},
-		{Type: iov1.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
+		{Type: ingresscontroller.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
 	}
 )
 
@@ -138,7 +139,7 @@ func TestUniqueDomainRejection(t *testing.T) {
 	defer assertIngressControllerDeleted(t, kclient, conflict)
 
 	conditions := []operatorv1.OperatorCondition{
-		{Type: iov1.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionFalse},
+		{Type: ingresscontroller.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionFalse},
 	}
 	err := waitForIngressControllerCondition(kclient, 5*time.Minute, conflictName, conditions...)
 	if err != nil {
@@ -557,7 +558,7 @@ func TestHostNetworkEndpointPublishingStrategy(t *testing.T) {
 	defer assertIngressControllerDeleted(t, kclient, ing)
 
 	conditions := []operatorv1.OperatorCondition{
-		{Type: iov1.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
+		{Type: ingresscontroller.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.IngressControllerAvailableConditionType, Status: operatorv1.ConditionTrue},
 		{Type: operatorv1.LoadBalancerManagedIngressConditionType, Status: operatorv1.ConditionFalse},
 		{Type: operatorv1.DNSManagedIngressConditionType, Status: operatorv1.ConditionFalse},
@@ -615,6 +616,29 @@ func TestInternalLoadBalancer(t *testing.T) {
 	}
 }
 
+// TestNodePortServiceEndpointPublishingStrategy creates an ingresscontroller
+// with the "NodePortService" endpoint publishing strategy type and verifies
+// that the operator creates a router and that the router becomes available.
+func TestNodePortServiceEndpointPublishingStrategy(t *testing.T) {
+	name := types.NamespacedName{Namespace: operatorNamespace, Name: "nodeport"}
+	ing := newNodePortController(name, name.Name+"."+dnsConfig.Spec.BaseDomain)
+	if err := kclient.Create(context.TODO(), ing); err != nil {
+		t.Fatalf("failed to create ingresscontroller: %v", err)
+	}
+	defer assertIngressControllerDeleted(t, kclient, ing)
+
+	conditions := []operatorv1.OperatorCondition{
+		{Type: ingresscontroller.IngressControllerAdmittedConditionType, Status: operatorv1.ConditionTrue},
+		{Type: operatorv1.IngressControllerAvailableConditionType, Status: operatorv1.ConditionTrue},
+		{Type: operatorv1.LoadBalancerManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+		{Type: operatorv1.DNSManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+	}
+	err := waitForIngressControllerCondition(kclient, 5*time.Minute, name, conditions...)
+	if err != nil {
+		t.Errorf("failed to observe expected conditions: %v", err)
+	}
+}
+
 // TestTLSSecurityProfile creates an ingresscontroller with no explicit TLS
 // profile, then verifies that the operator sets the default "Intermediate" TLS
 // profile, then updates the ingresscontroller to use a custom TLS profile, and
@@ -678,6 +702,120 @@ func TestTLSSecurityProfile(t *testing.T) {
 	}
 }
 
+func TestRouteAdmissionPolicy(t *testing.T) {
+	// Set up an ingresscontroller which only selects routes created by this test
+	icName := types.NamespacedName{Namespace: operatorNamespace, Name: "routeadmission"}
+	domain := icName.Name + "." + dnsConfig.Spec.BaseDomain
+	ic := newPrivateController(icName, domain)
+	ic.Spec.RouteAdmission = &operatorv1.RouteAdmissionPolicy{
+		NamespaceOwnership: operatorv1.StrictNamespaceOwnershipCheck,
+	}
+	ic.Spec.RouteSelector = &metav1.LabelSelector{
+		MatchLabels: map[string]string{
+			"routeadmissiontest": "",
+		},
+	}
+	if err := kclient.Create(context.TODO(), ic); err != nil {
+		t.Fatalf("failed to create ingresscontroller %s: %v", icName, err)
+	}
+	defer assertIngressControllerDeleted(t, kclient, ic)
+	conditions := []operatorv1.OperatorCondition{
+		{Type: operatorv1.IngressControllerAvailableConditionType, Status: operatorv1.ConditionTrue},
+		{Type: operatorv1.LoadBalancerManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+		{Type: operatorv1.DNSManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+	}
+	if err := waitForIngressControllerCondition(kclient, 5*time.Minute, icName, conditions...); err != nil {
+		t.Fatalf("failed to observe expected conditions: %v", err)
+	}
+
+	// Set up a pair of namespaces in which to create routes
+	ns1 := &corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "routeadmissionpolicytest1",
+		},
+	}
+	if err := kclient.Create(context.TODO(), ns1); err != nil {
+		t.Fatalf("failed to create namespace: %v", err)
+	}
+	defer func() {
+		if err := kclient.Delete(context.TODO(), ns1); err != nil {
+			t.Fatalf("failed to delete test namespace %v: %v", ns1.Name, err)
+		}
+	}()
+	ns2 := &corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: "routeadmissionpolicytest2",
+		},
+	}
+	if err := kclient.Create(context.TODO(), ns2); err != nil {
+		t.Fatalf("failed to create namespace: %v", err)
+	}
+	defer func() {
+		if err := kclient.Delete(context.TODO(), ns2); err != nil {
+			t.Fatalf("failed to delete test namespace %v: %v", ns2.Name, err)
+		}
+	}()
+
+	// Create conflicting routes in the namespaces
+	makeRoute := func(name types.NamespacedName, host, path string) *routev1.Route {
+		return &routev1.Route{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: name.Namespace,
+				Name:      name.Name,
+				Labels: map[string]string{
+					"routeadmissiontest": "",
+				},
+			},
+			Spec: routev1.RouteSpec{
+				Host: host,
+				Path: path,
+				To: routev1.RouteTargetReference{
+					Kind: "Service",
+					Name: "foo",
+				},
+			},
+		}
+	}
+	route1Name := types.NamespacedName{Namespace: ns1.Name, Name: "route"}
+	route1 := makeRoute(route1Name, "routeadmission.test.example.com", "/foo")
+
+	route2Name := types.NamespacedName{Namespace: ns2.Name, Name: "route"}
+	route2 := makeRoute(route2Name, "routeadmission.test.example.com", "/bar")
+
+	admittedCondition := routev1.RouteIngressCondition{Type: routev1.RouteAdmitted, Status: corev1.ConditionTrue}
+	rejectedCondition := routev1.RouteIngressCondition{Type: routev1.RouteAdmitted, Status: corev1.ConditionFalse}
+
+	// The first route should be admitted
+	if err := kclient.Create(context.TODO(), route1); err != nil {
+		t.Fatalf("failed to create route: %v", err)
+	}
+	if err := waitForRouteIngressConditions(kclient, route1Name, ic.Name, admittedCondition); err != nil {
+		t.Fatalf("failed to observe expected conditions: %v", err)
+	}
+
+	// The second route should be rejected because the policy is Strict
+	if err := kclient.Create(context.TODO(), route2); err != nil {
+		t.Fatalf("failed to create route: %v", err)
+	}
+	if err := waitForRouteIngressConditions(kclient, route2Name, ic.Name, rejectedCondition); err != nil {
+		t.Fatalf("failed to observe expected conditions: %v", err)
+	}
+
+	// Update the ingresscontroller to a different route admission policy
+	if err := kclient.Get(context.TODO(), icName, ic); err != nil {
+		t.Fatalf("failed to get ingresscontroller: %v", ic)
+	}
+	ic.Spec.RouteAdmission.NamespaceOwnership = operatorv1.InterNamespaceAllowedOwnershipCheck
+	if err := kclient.Update(context.TODO(), ic); err != nil {
+		t.Fatalf("failed to update ingresscontroller: %v", err)
+	}
+
+	// The second route should eventually be admitted because of the new policy
+	if err := waitForRouteIngressConditions(kclient, route2Name, ic.Name, admittedCondition); err != nil {
+		t.Fatalf("failed to observe expected conditions: %v", err)
+	}
+}
+
 func newLoadBalancerController(name types.NamespacedName, domain string) *operatorv1.IngressController {
 	repl := int32(1)
 	return &operatorv1.IngressController{
@@ -695,6 +833,23 @@ func newLoadBalancerController(name types.NamespacedName, domain string) *operat
 	}
 }
 
+func newNodePortController(name types.NamespacedName, domain string) *operatorv1.IngressController {
+	repl := int32(1)
+	return &operatorv1.IngressController{
+		ObjectMeta: metav1.ObjectMeta{
+			Namespace: name.Namespace,
+			Name:      name.Name,
+		},
+		Spec: operatorv1.IngressControllerSpec{
+			Domain:   domain,
+			Replicas: &repl,
+			EndpointPublishingStrategy: &operatorv1.EndpointPublishingStrategy{
+				Type: operatorv1.NodePortServiceStrategyType,
+			},
+		},
+	}
+}
+
 func newHostNetworkController(name types.NamespacedName, domain string) *operatorv1.IngressController {
 	repl := int32(1)
 	return &operatorv1.IngressController{
@@ -757,6 +912,14 @@ func clusterOperatorConditionMap(conditions ...configv1.ClusterOperatorStatusCon
 }
 
 func operatorConditionMap(conditions ...operatorv1.OperatorCondition) map[string]string {
+	conds := map[string]string{}
+	for _, cond := range conditions {
+		conds[cond.Type] = string(cond.Status)
+	}
+	return conds
+}
+
+func routeConditionMap(conditions ...routev1.RouteIngressCondition) map[string]string {
 	conds := map[string]string{}
 	for _, cond := range conditions {
 		conds[string(cond.Type)] = string(cond.Status)
@@ -787,6 +950,25 @@ func waitForClusterOperatorConditions(cl client.Client, conditions ...configv1.C
 	})
 }
 
+func waitForRouteIngressConditions(cl client.Client, routeName types.NamespacedName, routerName string, conditions ...routev1.RouteIngressCondition) error {
+	return wait.PollImmediate(1*time.Second, 10*time.Second, func() (bool, error) {
+		route := &routev1.Route{}
+		if err := cl.Get(context.TODO(), routeName, route); err != nil {
+			return false, err
+		}
+
+		for _, ingress := range route.Status.Ingress {
+			if ingress.RouterName == routerName {
+				expected := routeConditionMap(conditions...)
+				current := routeConditionMap(ingress.Conditions...)
+				return conditionsMatchExpected(expected, current), nil
+			}
+		}
+
+		return false, nil
+	})
+}
+
 func waitForIngressControllerCondition(cl client.Client, timeout time.Duration, name types.NamespacedName, conditions ...operatorv1.OperatorCondition) error {
 	return wait.PollImmediate(1*time.Second, timeout, func() (bool, error) {
 		ic := &operatorv1.IngressController{}
